@@ -13,6 +13,7 @@ from .constants import DEFAULT_REQUESTS_TIMEOUT
 from .facebook_scraper import FacebookScraper
 from .fb_types import Credentials, Post, RawPost, Profile
 from .utils import html_element_to_string, parse_cookie_file
+from . import exceptions
 
 
 _scraper = FacebookScraper()
@@ -23,8 +24,14 @@ def set_cookies(cookies):
     elif isinstance(cookies, dict):
         cookies = cookiejar_from_dict(cookies)
     if cookies is not None:
+        missing_cookies = [c for c in ['c_user', 'xs'] if c not in cookies]
+        if missing_cookies:
+            raise exceptions.InvalidCookies(f"Missing cookies with name(s): {missing_cookies}")
         _scraper.session.cookies = cookies
 
+def set_proxy(proxy):
+    _scraper.set_proxy(proxy)
+
 def get_profile(
     account: str,
     **kwargs,
@@ -40,6 +47,21 @@ def get_profile(
     set_cookies(cookies)
     return _scraper.get_profile(account, **kwargs)
 
+def get_group_info(
+    group: Union[str, int],
+    **kwargs
+) -> Profile:
+    """Get a group's profile information
+    Args:
+        group(str or int): The group name or ID
+        cookies (Union[dict, CookieJar, str]): Cookie jar to use.
+            Can also be a filename to load the cookies from a file (Netscape format).
+    """
+    _scraper.requests_kwargs['timeout'] = kwargs.pop('timeout', DEFAULT_REQUESTS_TIMEOUT)
+    cookies = kwargs.pop('cookies', None)
+    set_cookies(cookies)
+    return _scraper.get_group_info(group, **kwargs)
+
 def get_posts(
     account: Optional[str] = None,
     group: Union[str, int, None] = None,
