@@ -1,11 +1,13 @@
 import itertools
 import json
+import demjson
 import logging
 import re
 from datetime import datetime
 from json import JSONDecodeError
 from typing import Any, Dict, Optional
 from urllib.parse import parse_qs, urlparse
+from tqdm.auto import tqdm
 
 from . import utils
 from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL
@@ -32,6 +34,9 @@ def extract_post(raw_post: RawPost, options: Options, request_fn: RequestFunctio
 def extract_group_post(raw_post: RawPost, options: Options, request_fn: RequestFunction) -> Post:
     return GroupPostExtractor(raw_post, options, request_fn).extract_post()
 
+def extract_photo_post(raw_post: RawPost, options: Options, request_fn: RequestFunction) -> Post:
+    return PhotoPostExtractor(raw_post, options, request_fn).extract_post()
+
 
 class PostExtractor:
     """Class for Extracting fields from a FacebookPost"""
@@ -69,6 +74,8 @@ class PostExtractor:
         self.request = request_fn
 
         self._data_ft = None
+        self._full_post_html = None
+        self._live_data = {}
 
     # TODO: This is getting ugly, create a dataclass for Post
     def make_new_post(self) -> Post:
@@ -85,8 +92,14 @@ class PostExtractor:
             'images_lowquality': None,
             'images_lowquality_description': None,
             'video': None,
-            'video_thumbnail': None,
+            'video_duration_seconds': None,
+            'video_height': None,
             'video_id': None,
+            'video_quality': None,
+            'video_size_MB': None,
+            'video_thumbnail': None,
+            'video_watches': None,
+            'video_width': None,
             'likes': None,
             'comments': None,
             'shares': None,
@@ -113,6 +126,7 @@ class PostExtractor:
         """Parses the element into self.item"""
 
         methods = [
+            self.extract_post_url,
             self.extract_post_id,
             self.extract_text,
             self.extract_time,
@@ -121,13 +135,13 @@ class PostExtractor:
             self.extract_likes,
             self.extract_comments,
             self.extract_shares,
-            self.extract_post_url,
             self.extract_link,
             self.extract_user_id,
             self.extract_username,
             self.extract_video,
             self.extract_video_thumbnail,
             self.extract_video_id,
+            self.extract_video_meta,
             self.extract_is_live,
             self.extract_factcheck,
             self.extract_share_information,
@@ -183,7 +197,7 @@ class PostExtractor:
         return {'post_id': self.data_ft.get('top_level_post_id')}
 
     def extract_username(self) -> PartialPost:
-        elem = self.element.find('h3 strong a', first=True)
+        elem = self.element.find('h3 strong a,a.actor-link', first=True)
         if elem:
             url = elem.attrs.get("href")
             if url:
@@ -199,12 +213,8 @@ class PostExtractor:
         element = self.element
 
         has_more = self.more_url_regex.search(element.html)
-        if has_more and self.options.get("allow_extra_requests", True):
-            match = self.post_story_regex.search(element.html)
-            if match:
-                url = utils.urljoin(FB_MOBILE_BASE_URL, match.groups()[0].replace("&amp;", "&"))
-                response = self.request(url)
-                element = response.html.find('.story_body_container', first=True)
+        if has_more and self.full_post_html:
+            element = self.full_post_html.find('.story_body_container', first=True)
 
         nodes = element.find('p, header, span[role=presentation]')
         if nodes:
@@ -279,21 +289,32 @@ class PostExtractor:
         return {'user_id': self.data_ft['content_owner_id_new']}
 
     def extract_image_lq(self) -> PartialPost:
-        elems = self.element.find('div.story_body_container>div .img:not(.profpic):not([style*="static.xx.fbcdn.net"])')
+        elems = self.element.find('div.story_body_container>div .img:not(.profpic)')
+        if not elems:
+            elems = self.element.find('#root .img:not(.profpic)')
         images = []
         descriptions = []
         for elem in elems:
+            url = None
             if elem.attrs.get('src'):
-                images.append(elem.attrs.get('src'))
+                url = elem.attrs.get('src')
             elif elem.attrs.get('style'):
                 match = self.image_regex_lq.search(elem.attrs.get('style'))
                 if match:
-                    src = utils.decode_css_url(match.groups()[0])
-                    images.append(src)
-            descriptions.append(elem.attrs.get("aria-label") or elem.attrs.get("alt"))
+                    url = utils.decode_css_url(match.groups()[0])
+            if url and "static.xx.fbcdn.net" not in url:
+                images.append(url)
+                descriptions.append(elem.attrs.get("aria-label") or elem.attrs.get("alt"))
 
         image = images[0] if images else None
-        return {"image_lowquality": image, "images_lowquality": images, "images_lowquality_description": descriptions}
+        result = {"image_lowquality": image, "images_lowquality": images, "images_lowquality_description": descriptions}
+        # Link to high resolution external image embedded in low quality image url
+        if image and "safe_image.php" in image and not self.post.get("image"):
+            url = parse_qs(urlparse(image).query).get("url")
+            if url:
+                url = url[0]
+                result.update({"image": url, "images": [url]})
+        return result
 
     def extract_link(self) -> PartialPost:
         match = self.link_regex.search(self.element.html)
@@ -317,6 +338,7 @@ class PostExtractor:
 
             if post_match:
                 path = utils.filter_query_params(href, whitelist=query_params)
+                break
 
             elif video_post_match:
                 video_post_id = video_post_match.group(1)
@@ -326,7 +348,7 @@ class PostExtractor:
                 else:
                     path = f'{account}/videos/{video_post_id}'
 
-        post_id = self._data_ft.get('top_level_post_id')
+        post_id = self.data_ft.get('top_level_post_id')
 
         if video_post_match is None and account is not None and post_id is not None:
             path = f'{account}/posts/{post_id}'
@@ -343,6 +365,8 @@ class PostExtractor:
             'likes': utils.find_and_search(
                 self.element, 'footer', self.likes_regex, utils.convert_numeric_abbr
             )
+            or self.live_data.get("like_count")
+            or self.live_data.get("reactioncount")
             or 0,
         }
 
@@ -351,6 +375,7 @@ class PostExtractor:
             'comments': utils.find_and_search(
                 self.element, 'footer', self.comments_regex, utils.convert_numeric_abbr
             )
+            or self.live_data.get("comment_count")
             or 0,
         }
 
@@ -359,12 +384,13 @@ class PostExtractor:
             'shares': utils.find_and_search(
                 self.element, 'footer', self.shares_regex, utils.convert_numeric_abbr
             )
+            or self.live_data.get("share_count")
             or 0,
         }
 
-    def extract_photo_link_HQ(self, response: Response) -> URL:
+    def extract_photo_link_HQ(self, html: str) -> URL:
         # Find a link that says "View Full Size"
-        match = self.image_regex.search(response.text)
+        match = self.image_regex.search(html)
         if match:
             url = match.groups()[0].replace("&amp;", "&")
             if not url.startswith("http"):
@@ -372,9 +398,11 @@ class PostExtractor:
             if url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "/photo/view_full_size/")):
                 # Try resolve redirect
                 logger.debug(f"Fetching {url}")
-                redirect_response = self.request(url)
-                if not redirect_response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login.php")):
+                try:
+                    redirect_response = self.request(url)
                     url = redirect_response.html.find("a", first=True).attrs.get("href").replace("&amp;", "&")
+                except Exception as e:
+                    logger.error(e)
             return url
         else:
             return None
@@ -384,7 +412,7 @@ class PostExtractor:
             return None
         images = []
         descriptions = []
-        photo_links = self.element.find("div.story_body_container>div a[href*='photo.php'],a[href*='/photos/']")
+        photo_links = self.element.find("div.story_body_container>div a[href*='photo.php'], div.story_body_container>div a[href*='/photos/']")
         total_photos_in_gallery = len(photo_links)
         if len(photo_links) == 4 and photo_links[-1].text:
             total_photos_in_gallery = 4 + int(photo_links[-1].text.strip("+")) - 1
@@ -402,7 +430,7 @@ class PostExtractor:
             url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
             response = self.request(url)
-            images.append(self.extract_photo_link_HQ(response))
+            images.append(self.extract_photo_link_HQ(response.text))
             elem = response.html.find(".img[data-sigil='photo-image']", first=True)
             descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
 
@@ -413,7 +441,7 @@ class PostExtractor:
                 url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
             response = self.request(url)
-            images.append(self.extract_photo_link_HQ(response))
+            images.append(self.extract_photo_link_HQ(response.text))
             elem = response.html.find(".img[data-sigil='photo-image']", first=True)
             descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
         image = images[0] if images else None
@@ -434,45 +462,40 @@ class PostExtractor:
             print(more_info_post)
         ```
         """
+        reactions = {}
+
+        reaction_lookup = self.get_jsmod("UFIReactionTypes")
+        if reaction_lookup:
+            reaction_lookup = reaction_lookup.get("reactions")
+            for k,v in self.live_data.get("reactioncountmap").items():
+                if v["default"]:
+                    name = reaction_lookup[k]["display_name"].lower()
+                    reactions[name] = v["default"]
+
         url = self.post.get('post_url')
         post_id = self.post.get('post_id')
         w3_fb_url = url and utils.urlparse(url)._replace(netloc='www.facebook.com').geturl()
 
-        reaction_url = f'https://m.facebook.com/ufi/reaction/profile/browser/?ft_ent_identifier={post_id}'
-        response = self.request(reaction_url)
-        reactions = {}
-
-        # Dict mapping class names to human readable reaction names. Prepopulated in case FB doesn't include them
-        reaction_lookup = {
-            'sx_cbd149': 'Like',
-            'sx_202991': 'Love',
-            'sx_41edbc': 'Care',
-            'sx_0d839a': 'Haha',
-            'sx_2b1a8e': 'Wow',
-            'sx_454e38': 'Angry',
-            'sx_1a0b4b': 'Sad'
-        }
-
-        for span in response.html.find("span[aria-label]"):
-            label = span.attrs.get("aria-label", "")
-            if " people reacted with " in label:
-                reaction_count, reaction_type = label.split(" people reacted with ")
-                reactions[reaction_type.lower()] = utils.convert_numeric_abbr(reaction_count)
-                if self.options.get("reactors"):
-                    emoji_class = span.find("i", first=True).attrs.get("class")[-1]
-                    reaction_lookup[emoji_class] = reaction_type
-
         reactors = []
-
         reactors_opt = self.options.get("reactors")
         if reactors_opt:
+            reaction_url = f'https://m.facebook.com/ufi/reaction/profile/browser/?ft_ent_identifier={post_id}'
+            response = self.request(reaction_url)
+            emoji_class_lookup = {}
+            spriteMapCssClass = "sp_E24l_TeOlgh"
+            for k, v in self.get_jsmod("UFIReactionIcons").items():
+                name = reaction_lookup[k]["display_name"].lower()
+                for item in v.values():
+                    emoji_class_lookup[item["spriteCssClass"]] = name
+                    spriteMapCssClass = item["spriteMapCssClass"]
+
             """Fetch people reacting to an existing post obtained by `get_posts`.
             Note that this method may raise one more http request per post to get all reactors"""
             limit = 3000
             if type(reactors_opt) in [int, float] and reactors_opt < limit:
                 limit = reactors_opt
             logger.debug(f"Fetching {limit} reactors")
-            elems = list(response.html.find("div#reaction_profile_browser>div"))
+            elems = list(response.html.find("div#reaction_profile_browser>div,div#reaction_profile_browser1>div"))
             more = response.html.find("div#reaction_profile_pager a", first=True)
             if more and limit > 50:
                 url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
@@ -485,17 +508,17 @@ class PostExtractor:
                 for action in data['payload']['actions']:
                     if action['cmd'] == 'append':
                         html = utils.make_html_element(f"<div id='reaction_profile_browser'>{action['html']}</div>", url=FB_MOBILE_BASE_URL)
-                        more_elems = html.find('div#reaction_profile_browser>div')
+                        more_elems = html.find('div#reaction_profile_browser>div,div#reaction_profile_browser1>div')
                         elems.extend(more_elems)
             logger.debug(f"Found {len(elems)} reactors")
             for elem in elems:
-                emoji_class = elem.find("div>i:not(.nub)", first=True).attrs.get("class")[-1]
-                if not reaction_lookup.get(emoji_class):
+                emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get("class")[-1]
+                if not emoji_class_lookup.get(emoji_class):
                     logger.error(f"Don't know {emoji_class}")
                 reactors.append({
                     "name": elem.find("strong", first=True).text,
                     "link": utils.urljoin(FB_BASE_URL, elem.find("a", first=True).attrs.get("href")),
-                    "type": reaction_lookup.get(emoji_class)
+                    "type": emoji_class_lookup.get(emoji_class)
                 })
 
         if reactions:
@@ -590,6 +613,30 @@ class PostExtractor:
             return {'video_id': match.groups()[0]}
         return None
 
+    def extract_video_meta(self):
+        elem = self.full_post_html.find("script[type='application/ld+json']", first=True)
+        if not elem:
+            return None
+        meta = json.loads(elem.text)
+        if meta.get("@type") != "VideoObject":
+            return
+        watches = 0
+        if "interactionStatistic" in meta:
+            for interaction in meta["interactionStatistic"]:
+                if interaction.get("interactionType")["@type"] == 'http://schema.org/WatchAction':
+                    watches = interaction.get("userInteractionCount")
+        contentSize = None
+        if meta.get("contentSize"):
+            contentSize = float(meta['contentSize'].strip("kB")) / 1000
+        return {
+            'video_duration_seconds': utils.parse_duration(meta.get("duration")),
+            'video_watches': watches,
+            'video_quality': meta.get('videoQuality'),
+            'video_width': meta.get('width'),
+            'video_height': meta.get('height'),
+            'video_size_MB': contentSize,
+        }
+
     def extract_is_live(self):
         header = self.element.find('header')[0].full_text
 
@@ -659,12 +706,15 @@ class PostExtractor:
         # Try to extract from the abbr element
         date_element = comment.find('abbr', first=True)
         if date_element:
-            date = utils.parse_datetime(date_element.text, search=False)
+            date = utils.parse_datetime(date_element.text, search=True)
+            if not date:
+                logger.debug(f"Unable to parse {date_element.text}")
         else:
             date = None
 
         return {
             "comment_id": comment_id,
+            "comment_url": utils.urljoin(FB_BASE_URL, comment_id),
             "commenter_url": url,
             "commenter_name": name,
             "commenter_meta": commenter_meta,
@@ -675,41 +725,76 @@ class PostExtractor:
     def extract_comments_full(self):
         """Fetch comments for an existing post obtained by `get_posts`.
         Note that this method may raise multiple http requests per post to get all comments"""
-        url = self.post.get('post_url').replace(FB_BASE_URL, FB_MOBILE_BASE_URL)
-        logger.debug(f"Fetching {url}")
-        response = self.request(url)
-        elem = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
+        elem = self.full_post_html.find('div[data-sigil="m-mentions-expand"]', first=True)
         if not elem:
             logger.warning("No comments found on page")
             return
         comments = list(elem.find('div[data-sigil="comment"]'))
-        limit = 5000
+
+        direction = "View more comments"
+        more = elem.find("a", containing=direction, first=True)
+        if not more:
+            direction = "View previous comments"
+            more = elem.find("a", containing=direction, first=True)
+
+        # Comment limiting and progress
+        limit = 5000 # Default
+        if more and more.attrs.get("data-ajaxify-href"):
+            parsed = parse_qs(urlparse(more.attrs.get("data-ajaxify-href")).query)
+            count = int(parsed.get("count")[0])
+            if count < limit:
+                limit = count
         comments_opt = self.options.get('comments')
-        if type(comments_opt) in [int, float]:
+        if type(comments_opt) in [int, float] and comments_opt < limit:
             limit = comments_opt
-        more = elem.find("a", containing="View more comments", first=True)
-        while more and len(comments) < limit:
+        logger.debug(f"Fetching up to {limit} comments")
+
+        if self.options.get("progress"):
+            pbar = tqdm(total=limit)
+
+        visited_urls = []
+        while more and len(comments) <= limit:
             url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
-            logger.debug(f"Fetching {url}")
-            response = self.request(url)
+            if url in visited_urls:
+                logger.debug("cycle detected, break")
+                break
+            if self.options.get("progress"):
+                pbar.update(30)
+            else:
+                logger.debug(f"Fetching {url}")
+            try:
+                response = self.request(url)
+            except Exception as e:
+                logger.error(e)
+                break
+            visited_urls.append(url)
             elem = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
             if not elem:
                 logger.warning("No comments found on page")
                 break
             more_comments = elem.find('div[data-sigil="comment"]')
             comments.extend(more_comments)
-            more = response.html.find("a", containing="View more comments", first=True)
+            more = elem.find("a", containing=direction, first=True)
 
         logger.debug(f"Found {len(comments)} comments")
         results = []
+        if self.options.get("progress"):
+            pbar.close()
+            pbar = tqdm(total=len(comments))
         for comment in comments:
             result = self.parse_comment(comment)
             replies = comment.find("div.async_elem[data-sigil='replies-see-more'] a[href]", first=True)
+            if self.options.get("progress"):
+                pbar.update(1)
             if replies:
-                logger.debug(f"{result['comment_id']} has replies")
                 url = utils.urljoin(FB_MOBILE_BASE_URL, replies.attrs["href"])
-                logger.debug(f"Fetching {url}")
-                response = self.request(url)
+                if not self.options.get("progress"):
+                    logger.debug(f"Fetching {url}")
+                try:
+                    response = self.request(url)
+                except Exception as e:
+                    logger.error(e)
+                    continue
                 replies = response.html.find('div[data-sigil="comment"]')
                 result["replies"] = [self.parse_comment(reply) for reply in replies[1:]] # Skip first element, as it will be this comment itself
             results.append(result)
@@ -737,8 +822,64 @@ class PostExtractor:
 
         return self._data_ft
 
+    @property
+    def full_post_html(self):
+        if self._full_post_html is not None:
+            return self._full_post_html
+
+        if self.options.get("allow_extra_requests", True):
+            url = self.post.get('post_url').replace(FB_BASE_URL, FB_MOBILE_BASE_URL)
+            logger.debug(f"Fetching {url}")
+            response = self.request(url)
+            self._full_post_html = response.html
+            return self._full_post_html
+        else:
+            return None
+
+    @property
+    def live_data(self):
+        if self._live_data:
+            return self._live_data
+        try:
+            self._live_data = self.get_jsmod("MLiveData")
+        except Exception as e:
+            logger.error(e)
+        return self._live_data
+
+    def get_jsmod(self, name):
+        match = re.search(name + r'[^{]+({.+?})(?:\]\]|,\d)', self.full_post_html.html)
+        if match:
+            # Use demjson to load JS, as unquoted keys is not valid JSON
+            return demjson.decode(match.group(1))
+
 
 class GroupPostExtractor(PostExtractor):
     """Class for extracting posts from Facebook Groups rather than Pages"""
     post_url_regex = re.compile(r'https://m.facebook.com/groups/[^/]+/permalink/')
     post_story_regex = re.compile(r'href="(https://m.facebook.com/groups/[^/]+/permalink/\d+/)')
+
+class PhotoPostExtractor(PostExtractor):
+    def extract_text(self) -> PartialPost:
+        text = self.element.find("div.msg", first=True).text
+        return {"text": text, "post_text": text}
+
+    def extract_photo_link(self) -> PartialPost:
+        image = self.extract_photo_link_HQ(self.element.html)
+        return {"image": image, "images": [image]}
+
+    def extract_user_id(self) -> PartialPost:
+        match = re.search(r'entity_id:(\d+),', self.element.html)
+        if match:
+            return {"user_id": match.group(1)}
+
+    def extract_post_url(self) -> PartialPost:
+        return {"post_url": utils.urljoin(FB_MOBILE_BASE_URL, str(self.live_data["ft_ent_identifier"]))}
+
+    def extract_post_id(self) -> PartialPost:
+        return {"post_id": str(self.live_data["ft_ent_identifier"])}
+
+    def extract_likes(self) -> PartialPost:
+        return {"likes": self.live_data["like_count"]}
+
+    def extract_comments(self) -> PartialPost:
+        return {"comments": self.live_data["comment_count"]}
\ No newline at end of file
