@@ -5,10 +5,12 @@ import textwrap
 from typing import Iterator, Optional, Union
 
 from requests.exceptions import HTTPError
+import warnings
 
 from . import utils
 from .constants import FB_MOBILE_BASE_URL
 from .fb_types import URL, Page, RawPage, RequestFunction, Response
+from . import exceptions
 
 
 logger = logging.getLogger(__name__)
@@ -33,7 +35,10 @@ def generic_iter_pages(start_url, page_parser_cls, request_fn: RequestFunction,
 
     while next_url:
         logger.debug("Requesting page from: %s", next_url)
-        response = request_fn(next_url)
+        try:
+            response = request_fn(next_url)
+        except HTTPError as e:
+            raise exceptions.NotFound(e) from None
 
         logger.debug("Parsing page response")
         parser = page_parser_cls(response)
@@ -150,7 +155,7 @@ class PageParser:
 class GroupPageParser(PageParser):
     """Class for parsing a single page of a group"""
 
-    cursor_regex_3 = re.compile(r'\shref="(\/groups\/[^"]+bac=[^"]+)"')  # for Group requests
+    cursor_regex_3 = re.compile(r'href[=:]"(\/groups\/[^"]+bac=[^"]+)"')  # for Group requests
 
     def get_next_page(self) -> Optional[URL]:
         next_page = super().get_next_page()
