@@ -1,6 +1,7 @@
 import codecs
 import re
-from datetime import datetime
+from datetime import datetime, timedelta
+import calendar
 from typing import Optional
 from urllib.parse import parse_qsl, unquote, urlencode, urljoin, urlparse, urlunparse
 
@@ -28,6 +29,20 @@ def convert_numeric_abbr(s):
         return int(float(s[:-1]) * mapping[s[-1].lower()])
     return int(s)
 
+def parse_duration(s) -> int:
+    match = re.search('T(?P<hours>\d+H)?(?P<minutes>\d+M)?(?P<seconds>\d+S)', s)
+    if match:
+        result = 0
+        for k,v in match.groupdict().items():
+            if v:
+                if k == 'hours':
+                    result += int(v.strip("H")) * 60 * 60
+                elif k == "minutes":
+                    result += int(v.strip("M")) * 60
+                elif k == "seconds":
+                    result += int(v.strip("S"))
+        return result
+
 def decode_css_url(url: str) -> str:
     url = re.sub(r'\\(..) ', r'\\x\g<1>', url)
     url, _ = codecs.unicode_escape_decode(url)
@@ -68,7 +83,7 @@ month = (
     r"Nov(?:ember)?|"
     r"Dec(?:ember)?"
 )
-day = (
+day_of_week = (
     r"Mon|"
     r"Tue|"
     r"Wed|"
@@ -80,7 +95,6 @@ day = (
 day_of_month = r"\d{1,2}"
 specific_date_md = f"(?:{month}) {day_of_month}" + r"(?:,? \d{4})?"
 specific_date_dm = f"{day_of_month} (?:{month})" + r"(?:,? \d{4})?"
-day_of_week = f"on ({day})"
 
 date = f"{specific_date_md}|{specific_date_dm}|Today|Yesterday"
 
@@ -89,15 +103,15 @@ minute = r"\d{2}"
 period = r"AM|PM|"
 
 exact_time = f"(?:{date}) at {hour}:{minute} ?(?:{period})"
-relative_time_years = r'\b\d{1,2} yrs?'
-relative_time_months = r'\b\d{1,2} mths?'
-relative_time_weeks = r'\b\d{1,2} wks?'
+relative_time_years = r'\b\d{1,2} yr'
+relative_time_months = r'\b\d{1,2} (?:mth|mo)'
+relative_time_weeks = r'\b\d{1,2} wk'
 relative_time_hours = r"\b\d{1,2} ?h(?:rs?)?"
 relative_time_mins = r"\b\d{1,2} ?mins?"
 relative_time = f"{relative_time_years}|{relative_time_months}|{relative_time_weeks}|{relative_time_hours}|{relative_time_mins}"
 
-datetime_regex = re.compile(fr"({exact_time}|{relative_time}|{day_of_week})", re.IGNORECASE)
-
+datetime_regex = re.compile(fr"({exact_time}|{relative_time})", re.IGNORECASE)
+day_of_week_regex = re.compile(fr"({day_of_week})", re.IGNORECASE)
 
 def parse_datetime(text: str, search=True) -> Optional[datetime]:
     """Looks for a string that looks like a date and parses it into a datetime object.
@@ -114,8 +128,15 @@ def parse_datetime(text: str, search=True) -> Optional[datetime]:
     """
     if search:
         time_match = datetime_regex.search(text)
+        dow_match = day_of_week_regex.search(text)
         if time_match:
-            text = time_match.group(0).replace("wk", "week").replace("mth", "month").replace("yr", "year")
+            text = time_match.group(0)
+        elif dow_match:
+            text = dow_match.group(0)
+            today = calendar.day_abbr[datetime.today().weekday()]
+            if text == today:
+                # Fix for dateparser misinterpreting "last Monday" as today if today is Monday
+                return dateparser.parse(text) - timedelta(days=7)
         else:
             return None
 
@@ -143,7 +164,8 @@ def parse_cookie_file(filename: str) -> RequestsCookieJar:
                         expires = int(expires)
                     if "Name raw" in c:
                         # Cookie Quick Manager JSON format
-                        jar.set(c["Name raw"], c["Content raw"], domain=c["Host raw"], path=c["Path raw"], expires=expires)
+                        host = c["Host raw"].replace("https://", "").strip("/")
+                        jar.set(c["Name raw"], c["Content raw"], domain=host, path=c["Path raw"], expires=expires)
                     else:
                         # EditThisCookie JSON format
                         jar.set(c["name"], c["value"], domain=c["domain"], path=c["path"], secure=c["secure"], expires=expires)
